@@ -12,7 +12,6 @@ from __future__ import absolute_import
 from __future__ import with_statement
 
 import socket
-import sys
 
 from collections import deque
 from functools import partial
@@ -104,6 +103,7 @@ def itermessages(conn, channel, queue, limit=1, timeout=None,
     return drain_consumer(Consumer(channel, queues=[queue], **kwargs),
                           limit=limit, timeout=timeout, callbacks=callbacks)
 
+
 def eventloop(conn, limit=None, timeout=None, ignore_timeouts=False):
     """Best practice generator wrapper around ``Connection.drain_events``.
 
@@ -176,7 +176,7 @@ def collect_replies(conn, channel, queue, *args, **kwargs):
 def _ensure_errback(exc, interval):
     insured_logger.error(
         "Connection error: %r. Retry in %ss\n" % (exc, interval),
-            exc_info=sys.exc_info())
+            exc_info=True)
 
 
 def revive_connection(connection, channel, on_revive=None):
