@@ -11,20 +11,20 @@ Broker connection and pools.
 from __future__ import absolute_import
 from __future__ import with_statement
 
+import errno
 import os
-import sys
 import socket
 
 from contextlib import contextmanager
-from copy import copy
-from functools import partial, wraps
+from functools import partial
 from itertools import count
 from urllib import quote
 from Queue import Empty
 # jython breaks on relative import for .exceptions for some reason
 # (Issue #112)
 from kombu import exceptions
-from .transport import get_transport_cls
+from .log import get_logger
+from .transport import AMQP_ALIAS, get_transport_cls
 from .utils import cached_property, retry_over_time
 from .utils.compat import OrderedDict, LifoQueue as _LifoQueue
 from .utils.url import parse_url
@@ -34,6 +34,9 @@ _LOG_CHANNEL = os.environ.get("KOMBU_LOG_CHANNEL", False)
 
 __all__ = ["parse_url", "BrokerConnection", "Resource",
            "ConnectionPool", "ChannelPool"]
+URI_PASSTHROUGH = frozenset(["sqla", "sqlalchemy"])
+
+logger = get_logger(__name__)
 
 
 class BrokerConnection(object):
@@ -77,14 +80,17 @@ class BrokerConnection(object):
     _connection = None
     _default_channel = None
     _transport = None
-    _logger = None
-    uri_passthrough = set(["sqla", "sqlalchemy"])
+    _logger = False
     uri_prefix = None
 
     #: The cache of declared entities is per connection,
     #: in case the server loses data.
     declared_entities = None
 
+    #: This is set to True if there is still more data to read
+    #: after a call to :meth:`drain_nowait`.
+    more_to_read = False
+
     def __init__(self, hostname="localhost", userid=None,
             password=None, virtual_host=None, port=None, insist=False,
             ssl=False, transport=None, connect_timeout=5,
@@ -97,7 +103,7 @@ class BrokerConnection(object):
                   "transport": transport, "connect_timeout": connect_timeout,
                   "login_method": login_method}
         if hostname and "://" in hostname \
-                and transport not in self.uri_passthrough:
+                and transport not in URI_PASSTHROUGH:
             if '+' in hostname[:hostname.index("://")]:
                 # e.g. sqla+mysql://root:masterkey@localhost/
                 params["transport"], params["hostname"] = hostname.split('+')
@@ -114,8 +120,7 @@ class BrokerConnection(object):
         self.transport_options = transport_options
 
         if _LOG_CONNECTION:  # pragma: no cover
-            from .log import get_logger
-            self._logger = get_logger("kombu.connection")
+            self._logger = True
 
         if uri_prefix:
             self.uri_prefix = uri_prefix
@@ -137,8 +142,8 @@ class BrokerConnection(object):
 
     def _debug(self, msg, ident="[Kombu connection:0x%(id)x] ", **kwargs):
         if self._logger:  # pragma: no cover
-            self._logger.debug((ident + unicode(msg)) % {"id": id(self)},
-                               **kwargs)
+            logger.debug((ident + unicode(msg)) % {"id": id(self)},
+                         **kwargs)
 
     def connect(self):
         """Establish connection to server immediately."""
@@ -166,6 +171,20 @@ class BrokerConnection(object):
         """
         return self.transport.drain_events(self.connection, **kwargs)
 
+    def drain_nowait(self, *args, **kwargs):
+        try:
+            self.drain_events(timeout=0)
+        except socket.timeout:
+            self.more_to_read = False
+            return False
+        except socket.error, exc:
+            if exc.errno in (errno.EAGAIN, errno.EINTR):
+                self.more_to_read = False
+                return False
+            raise
+        self.more_to_read = True
+        return True
+
     def maybe_close_channel(self, channel):
         try:
             channel.close()
@@ -198,7 +217,8 @@ class BrokerConnection(object):
     close = release
 
     def ensure_connection(self, errback=None, max_retries=None,
-            interval_start=2, interval_step=2, interval_max=30):
+            interval_start=2, interval_step=2, interval_max=30,
+            callback=None):
         """Ensure we have a connection to the server.
 
         If not retry establishing the connection with the settings
@@ -216,11 +236,15 @@ class BrokerConnection(object):
           for each retry.
         :keyword interval_max: Maximum number of seconds to sleep between
           each retry.
+        :keyword callback: Optional callback that is called for every
+           internal iteration (1 s)
+        :keyword callback: Optional callback that is called for every
+           internal iteration (1 s).
 
         """
         retry_over_time(self.connect, self.connection_errors, (), {},
                         errback, max_retries,
-                        interval_start, interval_step, interval_max)
+                        interval_start, interval_step, interval_max, callback)
         return self
 
     def revive(self, new_channel):
@@ -228,6 +252,10 @@ class BrokerConnection(object):
             self.maybe_close_channel(self._default_channel)
             self._default_channel = None
 
+    def _default_ensure_callback(exc, interval):
+        logger.error("Ensure: Couldn't send message: %r. Retry in %ss",
+                     exc, interval, exc_info=True)
+
     def ensure(self, obj, fun, errback=None, max_retries=None,
             interval_start=1, interval_step=1, interval_max=1, on_revive=None):
         """Ensure operation completes, regardless of any channel/connection
@@ -264,7 +292,6 @@ class BrokerConnection(object):
 
         """
 
-        @wraps(fun)
         def _ensured(*args, **kwargs):
             got_connection = 0
             for retries in count(0):  # for infinity
@@ -272,7 +299,7 @@ class BrokerConnection(object):
                     return fun(*args, **kwargs)
                 except self.connection_errors + self.channel_errors, exc:
                     self._debug("ensure got exception: %r" % (exc, ),
-                                exc_info=sys.exc_info())
+                                exc_info=True)
                     if got_connection:
                         raise
                     if max_retries is not None and retries > max_retries:
@@ -294,8 +321,9 @@ class BrokerConnection(object):
                     if on_revive:
                         on_revive(new_channel)
                     got_connection += 1
-
         _ensured.func_name = _ensured.__name__ = "%s(ensured)" % fun.__name__
+        _ensured.__doc__ = fun.__doc__
+        _ensured.__module__ = fun.__module__
         return _ensured
 
     def autoretry(self, fun, channel=None, **ensure_options):
@@ -352,39 +380,41 @@ class BrokerConnection(object):
     def clone(self, **kwargs):
         """Create a copy of the connection with the same connection
         settings."""
-        return self.__class__(**dict(self.info(), **kwargs))
+        return self.__class__(**dict(self._info(), **kwargs))
 
-    def info(self):
-        """Get connection info."""
+    def _info(self):
         transport_cls = self.transport_cls or "amqp"
-        transport_cls = {"amqplib": "amqp"}.get(transport_cls, transport_cls)
-        defaults = self.transport.default_connection_params
+        transport_cls = {AMQP_ALIAS: "amqp"}.get(transport_cls, transport_cls)
+        D = self.transport.default_connection_params
         hostname = self.hostname
         if self.uri_prefix:
             hostname = "%s+%s" % (self.uri_prefix, hostname)
-        info = OrderedDict((("hostname", hostname),
-                            ("userid", self.userid),
-                            ("password", self.password),
-                            ("virtual_host", self.virtual_host),
-                            ("port", self.port),
-                            ("insist", self.insist),
-                            ("ssl", self.ssl),
-                            ("transport", transport_cls),
-                            ("connect_timeout", self.connect_timeout),
-                            ("transport_options", self.transport_options),
-                            ("login_method", self.login_method),
-                            ("uri_prefix", self.uri_prefix)))
-        for key, value in defaults.iteritems():
-            if info[key] is None:
-                info[key] = value
+        info = (("hostname", hostname or D.get("hostname")),
+                ("userid", self.userid or D.get("userid")),
+                ("password", self.password or D.get("password")),
+                ("virtual_host", self.virtual_host or D.get("virtual_host")),
+                ("port", self.port or D.get("port")),
+                ("insist", self.insist),
+                ("ssl", self.ssl),
+                ("transport", transport_cls),
+                ("connect_timeout", self.connect_timeout),
+                ("transport_options", self.transport_options),
+                ("login_method", self.login_method or D.get("login_method")),
+                ("uri_prefix", self.uri_prefix))
         return info
 
+    def info(self):
+        """Get connection info."""
+        return OrderedDict(self._info())
+
     def __eqhash__(self):
-        return hash("|".join(map(str, self.info().itervalues())))
+        return hash("%s|%s|%s|%s|%s|%s" % (
+            self.transport_cls, self.hostname, self.userid,
+            self.password, self.virtual_host, self.port))
 
     def as_uri(self, include_password=False):
-        if self.transport_cls in self.uri_passthrough:
-            return self.transport_cls + '+' + self.hostname
+        if self.transport_cls in URI_PASSTHROUGH:
+            return self.transport_cls + '+' + (self.hostname or "localhost")
         quoteS = partial(quote, safe="")   # strict quote
         fields = self.info()
         port = fields["port"]
@@ -610,6 +640,14 @@ class BrokerConnection(object):
     def channel_errors(self):
         """List of exceptions that may be raised by the channel."""
         return self.transport.channel_errors
+
+    @property
+    def eventmap(self):
+        return self.transport.eventmap(self.connection)
+
+    @property
+    def is_evented(self):
+        return getattr(self.transport, "on_poll_start", None)
 Connection = BrokerConnection
 
 
@@ -660,10 +698,16 @@ class Resource(object):
         else:
             R = self.prepare(self.new())
 
-        @wraps(self.release)
-        def _release():
+        def release():
+            """Release resource so it can be used by another thread.
+
+            The caller is responsible for discarding the object,
+            and to never use the resource again.  A new resource must
+            be acquired if so needed.
+
+            """
             self.release(R)
-        R.release = _release
+        R.release = release
 
         return R
 
@@ -684,13 +728,6 @@ class Resource(object):
         self.close_resource(resource)
 
     def release(self, resource):
-        """Release resource so it can be used by another thread.
-
-        The caller is responsible for discarding the object,
-        and to never use the resource again.  A new resource must
-        be acquired if so needed.
-
-        """
         if self.limit:
             self._dirty.discard(resource)
             self._resource.put_nowait(resource)
@@ -770,7 +807,7 @@ class ConnectionPool(Resource):
                                              preload=preload)
 
     def new(self):
-        return copy(self.connection)
+        return self.connection.clone()
 
     def release_resource(self, resource):
         resource._debug("released")
