@@ -5,6 +5,7 @@ import socket
 import types
 
 from anyjson import dumps
+from collections import defaultdict
 from itertools import count
 from Queue import Empty, Queue as _Queue
 
@@ -24,7 +25,7 @@ class _poll(eventio._select):
         events = []
         for fd in self._rfd:
             if fd.data:
-                events.append((fd.fileno(), eventio.POLL_READ))
+                events.append((fd.fileno(), eventio.READ))
         return events
 
 
@@ -38,7 +39,8 @@ class ResponseError(Exception):
 
 class Client(object):
     queues = {}
-    sets = {}
+    sets = defaultdict(set)
+    hashes = defaultdict(dict)
     shard_hint = None
 
     def __init__(self, db=None, port=None, **kwargs):
@@ -57,20 +59,28 @@ class Client(object):
     def delete(self, key):
         self.queues.pop(key, None)
 
-    def sadd(self, key, member):
-        print("SADD %r: %r" % (key, member))
-        if key not in self.sets:
-            self.sets[key] = set()
-        self.sets[key].add(member)
-
     def exists(self, key):
         return key in self.queues or key in self.sets
 
+    def hset(self, key, k, v):
+        self.hashes[key][k] = v
+
+    def hget(self, key, k):
+        return self.hashes[key].get(k)
+
+    def hdel(self, key, k):
+        self.hashes[key].pop(k, None)
+
+    def sadd(self, key, member, *args):
+        self.sets[key].add(member)
+    zadd = sadd
+
     def smembers(self, key):
         return self.sets.get(key, set())
 
-    def srem(self, key):
+    def srem(self, key, *args):
         self.sets.pop(key, None)
+    zrem = srem
 
     def llen(self, key):
         try:
@@ -271,7 +281,6 @@ class test_Channel(TestCase):
                               "data": "data"})
 
     def test_brpop_start_but_no_queues(self):
-        self.channel.active_queues.clear()
         self.assertIsNone(self.channel._brpop_start())
 
     def test_receive(self):
@@ -623,7 +632,7 @@ class test_MultiChannelPoller(TestCase):
 
     def test_close_unregisters_fds(self):
         p = self.Poller()
-        poller = p._poller = Mock()
+        poller = p.poller = Mock()
         p._chan_to_sock.update({1: 1, 2: 2, 3: 3})
 
         p.close()
@@ -637,14 +646,14 @@ class test_MultiChannelPoller(TestCase):
 
     def test_close_when_unregister_raises_KeyError(self):
         p = self.Poller()
-        p._poller = Mock()
+        p.poller = Mock()
         p._chan_to_sock.update({1: 1})
-        p._poller.unregister.side_effect = KeyError(1)
+        p.poller.unregister.side_effect = KeyError(1)
         p.close()
 
     def test_close_resets_state(self):
         p = self.Poller()
-        p._poller = Mock()
+        p.poller = Mock()
         p._channels = Mock()
         p._fd_to_chan = Mock()
         p._chan_to_sock = Mock()
@@ -656,21 +665,21 @@ class test_MultiChannelPoller(TestCase):
         p._channels.clear.assert_called_with()
         p._fd_to_chan.clear.assert_called_with()
         p._chan_to_sock.clear.assert_called_with()
-        self.assertIsNone(p._poller)
+        self.assertIsNone(p.poller)
 
     def test_register_when_registered_reregisters(self):
         p = self.Poller()
-        p._poller = Mock()
+        p.poller = Mock()
         channel, client, type = Mock(), Mock(), Mock()
         sock = client.connection._sock = Mock()
         sock.fileno.return_value = 10
 
         p._chan_to_sock = {(channel, client, type): 6}
         p._register(channel, client, type)
-        p._poller.unregister.assert_called_with(6)
+        p.poller.unregister.assert_called_with(6)
         self.assertTupleEqual(p._fd_to_chan[10], (channel, type))
         self.assertEqual(p._chan_to_sock[(channel, client, type)], sock)
-        p._poller.register.assert_called_with(sock, p.eventflags)
+        p.poller.register.assert_called_with(sock, p.eventflags)
 
         # when client not connected yet
         client.connection._sock = None
@@ -724,8 +733,8 @@ class test_MultiChannelPoller(TestCase):
         _aq = [] if queues is None else queues
         _af = [] if fanouts is None else fanouts
         p = self.Poller()
-        p._poller = Mock()
-        p._poller.poll.return_value = _pr
+        p.poller = Mock()
+        p.poller.poll.return_value = _pr
 
         p._register_BRPOP = Mock()
         p._register_LISTEN = Mock()
@@ -769,8 +778,8 @@ class test_MultiChannelPoller(TestCase):
 
         p._register_LISTEN.assert_called_with(channel)
 
-    def test_get_receives_POLL_ERR(self):
-        p, channel = self.create_get(events=[(1, eventio.POLL_ERR)])
+    def test_get_receives_ERR(self):
+        p, channel = self.create_get(events=[(1, eventio.ERR)])
         p._fd_to_chan[1] = (channel, "BRPOP")
 
         with self.assertRaises(redis.Empty):
@@ -779,8 +788,8 @@ class test_MultiChannelPoller(TestCase):
         channel._poll_error.assert_called_with("BRPOP")
 
     def test_get_receives_multiple(self):
-        p, channel = self.create_get(events=[(1, eventio.POLL_ERR),
-                                             (1, eventio.POLL_ERR)])
+        p, channel = self.create_get(events=[(1, eventio.ERR),
+                                             (1, eventio.ERR)])
         p._fd_to_chan[1] = (channel, "BRPOP")
 
         with self.assertRaises(redis.Empty):
