@@ -12,7 +12,19 @@ from __future__ import absolute_import
 
 import sys
 
-DEFAULT_TRANSPORT = "kombu.transport.amqplib.Transport"
+from kombu.syn import detect_environment
+
+DEFAULT_TRANSPORT = "amqp"
+
+AMQP_TRANSPORT = "kombu.transport.amqplib.Transport"
+AMQP_ALIAS = "librabbitmq"
+if detect_environment() == "default":
+    try:
+        import librabbitmq  # noqa
+        AMQP_TRANSPORT = "kombu.transport.librabbitmq.Transport"  # noqa
+        AMQP_ALIAS = "amqp"                                       # noqa
+    except ImportError:
+        pass
 
 
 def _ghettoq(name, new, alias=None):
@@ -36,7 +48,7 @@ def _ghettoq(name, new, alias=None):
 
 
 TRANSPORT_ALIASES = {
-    "amqp": "kombu.transport.amqplib.Transport",
+    "amqp": AMQP_TRANSPORT,
     "amqplib": "kombu.transport.amqplib.Transport",
     "librabbitmq": "kombu.transport.librabbitmq.Transport",
     "pika": "kombu.transport.pika2.Transport",
@@ -48,6 +60,7 @@ TRANSPORT_ALIASES = {
     "beanstalk": "kombu.transport.beanstalk.Transport",
     "mongodb": "kombu.transport.mongodb.Transport",
     "couchdb": "kombu.transport.couchdb.Transport",
+    "zookeeper": "kombu.transport.zookeeper.Transport",
     "django": "kombu.transport.django.Transport",
     "sqlalchemy": "kombu.transport.sqlalchemy.Transport",
     "sqla": "kombu.transport.sqlalchemy.Transport",
