@@ -189,6 +189,10 @@ class Transport(object):
     #: Tuple of errors that can happen due to channel/method failure.
     channel_errors = ()
 
+    #: For non-blocking use, an eventloop should keep
+    #: draining events as long as ``connection.more_to_read`` is True.
+    nb_keep_draining = False
+
     def __init__(self, client, **kwargs):
         self.client = client
 
@@ -207,6 +211,11 @@ class Transport(object):
     def drain_events(self, connection, **kwargs):
         raise NotImplementedError("Subclass responsibility")
 
+    def eventmap(self, connection):
+        """Map of fd -> event handler for event based use.
+        Unconvenient to use, and limited transport support."""
+        return {}
+
     def verify_connection(self, connection):
         return True
 
