@@ -2,17 +2,24 @@
 kombu.transport.librabbitmq
 ===========================
 
-pylibrabbitmq transport.
+`librabbitmq`_ transport.
+
+.. _`librabbitmq`: http://pypi.python.org/librabbitmq/
 
 :copyright: (c) 2010 - 2012 by Ask Solem.
 :license: BSD, see LICENSE for more details.
 
 """
 from __future__ import absolute_import
-import socket
-import pylibrabbitmq as amqp
 
-from pylibrabbitmq import ChannelError, ConnectionError
+import socket
+
+try:
+    import librabbitmq as amqp
+    from librabbitmq import ChannelError, ConnectionError
+except ImportError:
+    import pylibrabbitmq as amqp                             # noqa
+    from pylibrabbitmq import ChannelError, ConnectionError  # noqa
 
 from kombu.exceptions import StdChannelError
 
@@ -23,7 +30,7 @@ DEFAULT_PORT = 5672
 
 class Message(base.Message):
 
-    def __init__(self, body, props, info, channel):
+    def __init__(self, channel, props, info, body):
         super(Message, self).__init__(channel,
                 body=body,
                 delivery_info=info,
@@ -46,11 +53,12 @@ class Channel(amqp.Channel, base.StdChannel):
                            "content_encoding": content_encoding,
                            "headers": headers,
                            "priority": priority})
-        return amqp.Message(body, properties=properties)
+        return body, properties
 
 
 class Connection(amqp.Connection):
     Channel = Channel
+    Message = Message
 
 
 class Transport(base.Transport):
@@ -63,6 +71,8 @@ class Transport(base.Transport):
                          OSError)
     channel_errors = (StdChannelError, ChannelError, )
 
+    nb_keep_draining = True
+
     def __init__(self, client, **kwargs):
         self.client = client
         self.default_port = kwargs.get("default_port") or self.default_port
@@ -95,6 +105,15 @@ class Transport(base.Transport):
         """Close the AMQP broker connection."""
         connection.close()
 
+    def on_poll_init(self, poller):
+        pass
+
+    def on_poll_start(self):
+        return {}
+
+    def eventmap(self, connection):
+        return {connection.fileno(): self.client.drain_nowait}
+
     @property
     def default_connection_params(self):
         return {"userid": "guest", "password": "guest",
