@@ -94,7 +94,10 @@ class QoS(object):
     #: Instead of a mutex, which doesn't perform well here, we mark
     #: the delivery tags as dirty, so subsequent calls to append() can remove
     #: them.
-    _dirty = set()
+    _dirty = None
+
+    #: If disabled, unacked messages won't be restored at shutdown.
+    restore_at_shutdown = True
 
     def __init__(self, channel, prefetch_count=0):
         self.channel = channel
@@ -102,6 +105,7 @@ class QoS(object):
 
         self._delivered = OrderedDict()
         self._delivered.restored = False
+        self._dirty = set()
         self._on_collect = Finalize(self,
                                     self.restore_unacked_once,
                                     exitpriority=1)
@@ -166,7 +170,7 @@ class QoS(object):
         return errors
 
     def restore_unacked_once(self):
-        """Restores all uncknowledged message at shutdown/gc collect.
+        """Restores all unacknowledged message at shutdown/gc collect.
 
         Will only be done once for each instance.
 
@@ -175,13 +179,16 @@ class QoS(object):
         self._flush()
         state = self._delivered
 
-        if not self.channel.do_restore or getattr(state, "restored", None):
+        if not self.restore_at_shutdown:
+            return
+        elif not self.channel.do_restore or getattr(state, "restored", None):
             assert not state
             return
 
         try:
             if state:
-                say("Restoring unacknowledged messages: %s", state)
+                say("Restoring %r unacknowledged message(s).",
+                        len(self._delivered))
                 unrestored = self.restore_unacked()
 
                 if unrestored:
@@ -196,6 +203,7 @@ class QoS(object):
 class Message(base.Message):
 
     def __init__(self, channel, payload, **kwargs):
+        self._raw = payload
         properties = payload["properties"]
         body = payload.get("body")
         if body:
@@ -214,6 +222,9 @@ class Message(base.Message):
         props = self.properties
         body, _ = self.channel.encode_body(self.body,
                                            props.get("body_encoding"))
+        headers = dict(self.headers)
+        # remove compression header
+        headers.pop("compression", None)
         return {"body": body,
                 "properties": props,
                 "content-type": self.content_type,
@@ -258,7 +269,7 @@ class AbstractChannel(object):
     def _new_queue(self, queue, **kwargs):
         """Create new queue.
 
-        Some implementations needs to do additiona actions when
+        Some implementations needs to do additional actions when
         the queue is created.  You can do so by overriding this
         method.
 
@@ -288,6 +299,9 @@ class Channel(AbstractChannel, base.StdChannel):
     #: message class used.
     Message = Message
 
+    #: QoS class used.
+    QoS = QoS
+
     #: flag to restore unacked messages when channel
     #: goes out of scope.
     do_restore = True
@@ -365,7 +379,7 @@ class Channel(AbstractChannel, base.StdChannel):
             self.queue_delete(queue, if_unused=True, if_empty=True)
         self.state.exchanges.pop(exchange, None)
 
-    def queue_declare(self, queue, passive=False, auto_delete=False, **kwargs):
+    def queue_declare(self, queue, passive=False, **kwargs):
         """Declare queue."""
         if passive and not self._has_queue(queue, **kwargs):
             raise StdChannelError("404",
@@ -608,7 +622,7 @@ class Channel(AbstractChannel, base.StdChannel):
     def qos(self):
         """:class:`QoS` manager for this channel."""
         if self._qos is None:
-            self._qos = QoS(self)
+            self._qos = self.QoS(self)
         return self._qos
 
     @property
