@@ -19,11 +19,11 @@ def _parse_url(url):
     # use port but let pymongo get the port(s) from the URI instead.
     # This enables the use of replica sets and sharding.
     # See pymongo.Connection() for more info.
+    port = scheme != 'mongodb' and parts.port or None
     hostname = schemeless if scheme == 'mongodb' else parts.hostname
     path = parts.path or ''
     path = path[1:] if path and path[0] == '/' else path
-    return (scheme, unquote(hostname or '') or None,
-            int(parts.port) if parts.port else None,
+    return (scheme, unquote(hostname or '') or None, port,
             unquote(parts.username or '') or None,
             unquote(parts.password or '') or None,
             unquote(path or '') or None,
